package parse;

import org.xml.sax.Attributes;

import xcore.Cons;
import xcore.List;
import xcore.Nil;
import xcore.Str;
import xcore.Value;
import env.Env;

public class Call extends Parser {

  // Call a named rule in the grammar. The arguments are the values
  // variables that are bound at that point in the parse...

  private String name;
  private List<Str> args;

  public Call(Str name, List<Str> args) {
	super();
	this.name = name.getValue();
	this.args = args;
  }

  public String getName() {
	return name;
  }

  public void setName(String name) {
	this.name = name;
  }

  public List<Str> getArgs() {
	return args;
  }

  public void setArgs(List<Str> args) {
	this.args = args;
  }

  public TagProcessor parse(final Grammar grammar, final Success s, final Env<String, Value> env, Fail c, final Fail f) {
	Success success = new Success() {

	  public TagProcessor success(Object value, Env<String, Value> ignore, Fail fail) {
		grammar.undent();
		grammar.tab(System.out);
		grammar.debug("Return " + value);
		return s.success(value, env, fail);
	  }

	};
	Fail fail = new Fail() {

	  public TagProcessor fail(String tag, Attributes attributes, Boolean isStart) {
		grammar.undent();
		grammar.tab(System.out);
		grammar.debug("FAIL");
		return f.fail(tag, attributes, isStart);
	  }

	};
	grammar.tab(System.out);
	grammar.debug("Call " + name);
	grammar.indent();
	return grammar.call(success, name, values(args, env), c, fail);
  }

  private List<Value> values(List<Str> args, Env<String, Value> env) {
	if (args.isEmpty())
	  return new Nil<Value>();
	else {
	  Cons<Str> cons = (Cons<Str>) args;
	  Str arg = cons.getHead();
	  Value value = env.lookup(arg.getValue());
	  if (value == null)
		throw new Error("Unbound variable " + arg);
	  return new Cons<Value>(value, values(cons.getTail(), env));
	}
  }

}
