package parse;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import xcore.List;
import xcore.Str;
import xcore.Value;
import env.Env;

public class CnstrAction extends Action {

  // Create an instance of a Java class. The arguments to the
  // constructor are the values of variables bound in the parse.
  // A CnstrAction is created by supplying the Java path to the
  // class as a string and a sequence of the names of the grammar
  // variables.

  private Constructor<?> constructor;

  public CnstrAction(Constructor<?> constructor, List<Str> args) {
	super();
	this.constructor = constructor;
	this.setArgs(args);
  }

  public CnstrAction(Str className, List<Str> args) {
	this(getConstructor(namedClass(className.getValue()), args.size()), args);
  }

  public static Class<?> namedClass(String className) {
	try {
	  return java.lang.Class.forName(className);
	} catch (ClassNotFoundException e) {
	  throw new Error(e);
	}
  }

  public Constructor<?> getConstructor() {
	return constructor;
  }

  public void setConstructor(Constructor<?> constructor) {
	this.constructor = constructor;
  }

  private static Constructor<?> getConstructor(Class<?> c, int arity) {
	Constructor<?>[] constructors = c.getConstructors();
	for (Constructor<?> constructor : constructors)
	  if (okParams(constructor.getParameterTypes(), arity))
		return constructor;
	throw new Error("No constructors for " + c + " with arity " + arity);
  }

  private static boolean okParams(Class<?>[] types, int arity) {
	if (types.length == arity) {
	  for (int i = 0; i < arity; i++)
		if (!Value.class.isAssignableFrom(types[i]))
		  return false;
	  return true;
	} else
	  return false;
  }

  public TagProcessor parse(Grammar grammar, Success s, Env<String, Value> env, Fail c, Fail f) {
	try {
	  // Try to create a new instanceof the Java class...
	  grammar.tab(System.out);
	  grammar.debug("Cnstr " + constructor + " args = " + getArgs() + " env = " + env);
	  Object value = constructor.newInstance(consArgs(env));
	  return s.success(value, env, f);
	} catch (IllegalArgumentException e) {
	  throw new Error(constructor.toString() + env + getArgs() + e);
	} catch (InstantiationException e) {
	  throw new Error(e);
	} catch (IllegalAccessException e) {
	  throw new Error(e);
	} catch (InvocationTargetException e) {
	  throw new Error(e);
	}
  }

  private Object[] consArgs(Env<String, Value> env) {
	Object[] argValues = new Object[getArgs().size()];
	for (int i = 0; i < getArgs().size(); i++) {
	  Value value = env.lookup(getArgs().get(i).getValue());
	  if (value == null)
		throw new Error("Unbound variable " + getArgs().get(i));
	  argValues[i] = value;
	}
	return argValues;
  }

  public String toString() {
	return "CnstrAction(" + constructor + "," + getArgs() + ")";
  }

}
