package parse;

import java.io.PrintStream;

import xcore.Cons;
import xcore.IdBuilder;
import xcore.List;
import xcore.Nil;
import xcore.Resolver;
import xcore.Str;
import xcore.Value;
import xcore.Walker;

public class Grammar extends Value {

  // A grammar consists of a collection of rules. Each rule is named
  // has some arguments and a body that is a parser. A grammar processes
  // an XML tree and synthesizes a Java value.

  private boolean debug = false;
  private int indent = 0;
  private List<Rule> rules = new Nil<Rule>();

  public Grammar() {
  }

  public Grammar(List<Rule> rules) {
	this.rules = rules;
  }

  public List<Rule> getRules() {
	return rules;
  }

  public void tab(PrintStream printStream) {
	if (debug)
	  for (int i = 0; i < indent; i++)
		printStream.write(' ');
  }

  public void indent() {
	indent++;
  }

  public void undent() {
	indent--;
  }

  public void setRules(List<Rule> rules) {
	this.rules = rules;
  }

  public void addRule(Rule rule) {
	rules = new Cons<Rule>(rule, rules);
  }

  public TagProcessor call(Success s, String name, List<Value> values, Fail c, Fail f) {
	Rule rule = findRule(name, values.size());
	if (rule != null)
	  return rule.parse(this, s, values, c, f);
	else
	  throw new Error("No rule " + name + "/" + values.size());
  }

  public Rule findRule(String name, int length) {
	for (Rule rule : rules)
	  if (rule.getName().equals(name))
		return rule;
	return null;
  }

  public static Grammar grammar() {
	
	// The XML parser is bootstrapped. This gramar is sufficient in order
	// to parse the XML definition of itself.
	
	Grammar grammar = new Grammar();
	Rule Grammar = new Rule("Grammar", new Nil<Str>(), new Element(new Str("Grammar"), new And(new Bind(new Str("rules"), new Star(new Call(new Str("Rule"), new Nil<Str>()))),
	    new CnstrAction(new Str("parse.Grammar"), new Cons<Str>(new Str("rules"), new Nil<Str>()))), new Nil<Str>()));
	Rule Rule = new Rule("Rule", new Nil<Str>(), new Element(new Str("Rule"), new And(new Bind(new Str("args"), new Star(new Call(new Str("Arg"), new Nil<Str>()))), new Bind(
	    new Str("body"), new Call(new Str("Parser"), new Nil<Str>()))), new Cons<Str>(new Str("name"), new Nil<Str>())), new CnstrAction(new Str("parse.Rule"), new Cons<Str>(
	    new Str("name"), new Cons<Str>(new Str("args"), new Cons<Str>(new Str("body"), new Nil<Str>())))));
	Rule Arg = new Rule("Arg", new Nil<Str>(), new Element(new Str("Arg"), new Cons<Str>(new Str("name"), new Nil<Str>())), new VarRef(new Str("name")));
	Rule Att = new Rule("Att", new Nil<Str>(), new Element(new Str("Att"), new Cons<Str>(new Str("name"), new Nil<Str>())), new VarRef(new Str("name")));
	Rule Parser = new Rule("Parser", new Nil<Str>(), new Or(new Call(new Str("Or"), new Nil<Str>()), new Or(new Call(new Str("And"), new Nil<Str>()), new Or(new Call(new Str(
	    "Bind"), new Nil<Str>()), new Or(new Call(new Str("Call"), new Nil<Str>()), new Or(new Call(new Str("Cnstr"), new Nil<Str>()), new Or(new Call(new Str("Element"),
	    new Nil<Str>()), new Or(new Call(new Str("Star"), new Nil<Str>()), new Or(new Call(new Str("VarRef"), new Nil<Str>()), new Call(new Str("OK"), new Nil<Str>()))))))))));
	Rule OK = new Rule("OK", new Nil<Str>(), new Element(new Str("OK"), new CnstrAction(new Str("parse.OK"), new Nil<Str>()), new Nil<Str>()));
	Rule Bind = new Rule("Bind", new Nil<Str>(), new Element(new Str("Bind"), new And(new Bind(new Str("child"), new Call(new Str("Parser"), new Nil<Str>())), new CnstrAction(
	    new Str("parse.Bind"), new Cons<Str>(new Str("name"), new Cons<Str>(new Str("child"), new Nil<Str>())))), new Cons<Str>(new Str("name"), new Nil<Str>())));
	Rule VarRef = new Rule("VarRef", new Nil<Str>(), new Element(new Str("VarRef"), new CnstrAction(new Str("parse.VarRef"), new Cons<Str>(new Str("name"), new Nil<Str>())),
	    new Cons<Str>(new Str("name"), new Nil<Str>())));
	Rule Call = new Rule("Call", new Nil<Str>(), new Element(new Str("Call"), new And(new Bind(new Str("args"), new Star(new Call(new Str("Arg"), new Nil<Str>()))),
	    new CnstrAction(new Str("parse.Call"), new Cons<Str>(new Str("name"), new Cons<Str>(new Str("args"), new Nil<Str>())))), new Cons<Str>(new Str("name"), new Nil<Str>())));
	Rule Cnstr = new Rule("Cnstr", new Nil<Str>(), new Element(new Str("Cnstr"), new And(new Bind(new Str("args"), new Star(new Call(new Str("Arg"), new Nil<Str>()))),
	    new CnstrAction(new Str("parse.CnstrAction"), new Cons<Str>(new Str("name"), new Cons<Str>(new Str("args"), new Nil<Str>())))), new Cons<Str>(new Str("name"),
	    new Nil<Str>())));
	Rule And = new Rule("And", new Nil<Str>(), new Element(new Str("And"), new And(new And(new Bind(new Str("left"), new Call(new Str("Parser"), new Nil<Str>())), new Bind(
	    new Str("right"), new Call(new Str("Parser"), new Nil<Str>()))), new CnstrAction(new Str("parse.And"), new Cons<Str>(new Str("left"), new Cons<Str>(new Str("right"),
	    new Nil<Str>())))), new Nil<Str>()));
	Rule Or = new Rule("Or", new Nil<Str>(), new Element(new Str("Or"), new And(new And(new Bind(new Str("left"), new Call(new Str("Parser"), new Nil<Str>())), new Bind(new Str(
	    "right"), new Call(new Str("Parser"), new Nil<Str>()))), new CnstrAction(new Str("parse.Or"), new Cons<Str>(new Str("left"),
	    new Cons<Str>(new Str("right"), new Nil<Str>())))), new Nil<Str>()));
	Rule Element = new Rule("Element", new Nil<Str>(), new Element(new Str("Element"), new And(new Bind(new Str("child"), new Call(new Str("Parser"), new Nil<Str>())), new And(
	    new Bind(new Str("atts"), new Star(new Call(new Str("Att"), new Nil<Str>()))), new CnstrAction(new Str("parse.Element"), new Cons<Str>(new Str("tag"), new Cons<Str>(
	        new Str("child"), new Cons<Str>(new Str("atts"), new Nil<Str>())))))), new Cons<Str>(new Str("tag"), new Nil<Str>())));
	Rule Star = new Rule("Star", new Nil<Str>(), new Element(new Str("Star"), new And(new Bind(new Str("parser"), new Call(new Str("Parser"), new Nil<Str>())), new CnstrAction(
	    new Str("parse.Star"), new Cons<Str>(new Str("parser"), new Nil<Str>()))), new Nil<Str>()));
	grammar.addRule(Grammar);
	grammar.addRule(Rule);
	grammar.addRule(Arg);
	grammar.addRule(Att);
	grammar.addRule(Parser);
	grammar.addRule(OK);
	grammar.addRule(And);
	grammar.addRule(Or);
	grammar.addRule(Bind);
	grammar.addRule(Call);
	grammar.addRule(Cnstr);
	grammar.addRule(Element);
	grammar.addRule(Star);
	grammar.addRule(VarRef);
	return grammar;
  }

  public static void main(String[] args) {
	
	// The entry point to test the parsing system. The arguments are:
	// (1) The path to the definition of Grammar.xml
	// (2) The path to a grammar.
	// (3) The path to a data file.
	
	// Parse the definition of the XML grammar...
	
	Value v = new parse.XMLReader(grammar(), "Grammar", new Nil<Value>(), args[0]).parse();
	Printer printer = new Printer(System.out);
	// printer.walk(v);
	
	// Parse the definition of a test grammar...

	v = new parse.XMLReader((Grammar) v, "Grammar", new Nil<Value>(), args[1]).parse();
	printer = new Printer(System.out);
	// printer.walk(v);

	Grammar g = (Grammar) v;
	// g.setDebug(true);
	
	// Parse the definition of a value in the new language...
	
	v = new parse.XMLReader(g, "Value", new Nil<Value>(), args[2]).parse();
	printer = new Printer(System.out);
	IdBuilder ids = new IdBuilder();
	ids.walk(v);
	
	// There may be XML references that need to be resolved...
	
	Resolver r = new Resolver(ids.getTable());
	r.walk(v);
	
	// Define marks...
	xcore.Mark mark = new xcore.Mark();
	mark.walk(v);
	
	// System.out.println(v);
	printer.setMark(mark);
	printer.walk(v);
  }

  public void walk(Walker walker) {

  }

  public String toString() {
	String s = "Grammar(";
	for (Rule r : rules) {
	  s = s + r;
	  if (r != rules.get(rules.size() - 1))
		s = s + ",";
	}
	return s + ")";
  }

  public void debug(String string) {
	if (debug)
	  System.out.println(string);
  }

  public boolean isDebug() {
	return debug;
  }

  public void setDebug(boolean debug) {
	this.debug = debug;
  }

}
