package parse;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import xcore.List;
import xcore.Str;
import xcore.Value;
import env.Env;

public class MethodAction extends Action {

  // An action that calls a Java method...

  private Method method;

  public MethodAction(Method method, List<Str> args) {
	super();
	this.method = method;
	this.setArgs(args);
  }

  public MethodAction(Class<?> c, String name, List<Str> args) {
	this(getMethod(c, name), args);
  }

  public Method getMethod() {
	return method;
  }

  public void setMethod(Method method) {
	this.method = method;
  }

  private static Method getMethod(Class<?> c, String name) {
	Method[] methods = c.getMethods();
	for (Method method : methods)
	  if (method.getName().equals(name))
		return method;
	throw new Error("No methods named " + name);
  }

  public TagProcessor parse(Grammar grammar, Success s, Env<String, Value> env, Fail c, Fail f) {
	try {
	  Object value = method.invoke(null, (Object[]) consArgs(env));
	  return s.success(value, env, f);
	} catch (IllegalArgumentException e) {
	  throw new Error(e);
	} catch (IllegalAccessException e) {
	  throw new Error(e);
	} catch (InvocationTargetException e) {
	  throw new Error(e);
	}
  }

  private Value[] consArgs(Env<String, Value> env) {
	Value[] argValues = new Value[getArgs().size()];
	for (int i = 0; i < getArgs().size(); i++)
	  argValues[i] = env.lookup(getArgs().get(i).getValue());
	return argValues;
  }

}
