package parse;

import java.io.PrintStream;

import xcore.Str;

public class Printer extends xcore.Printer {

  // Printers are walkers that are extended for a
  // collection of classes. These are the walkers
  // for printing grammar elements...

  public Printer(PrintStream out) {
	super(out);
  }

  public void walk(Grammar grammar) {
	getOut().print("(Grammar)[rules=");
	for (Rule rule : grammar.getRules())
	  walk(rule);
	getOut().print("]");
  }

  public void walk(Rule rule) {
	getOut().print("(Rule)[name=");
	getOut().print(rule.getName());
	getOut().print(", args=[");
	for (Str arg : rule.getArgs()) {
	  getOut().print(arg.getValue());
	  if (arg != rule.getArgs().get(rule.getArgs().size() - 1))
		getOut().print(",");
	}
	getOut().print("],body=");
	walk(rule.getBody());
	getOut().print("]");
  }

  public void walk(OK ok) {
	getOut().print("OK");
  }

  public void walk(And and) {
	getOut().print("(And)[first=");
	walk(and.getFirst());
	getOut().print(",second=");
	walk(and.getSecond());
	getOut().print("]");
  }

  public void walk(Or and) {
	getOut().print("(Or)[Left=");
	walk(and.getLeft());
	getOut().print(",right=");
	walk(and.getRight());
	getOut().print("]");
  }

  public void walk(Bind bind) {
	getOut().print("(Bind)[name=");
	getOut().print(bind.getName());
	getOut().print(",parser=");
	walk(bind.getParser());
	getOut().print("]");
  }

  public void walk(Call call) {
	getOut().print("(Call)[name=");
	getOut().print(call.getName());
	getOut().print(",args=[");
	for (Str arg : call.getArgs()) {
	  getOut().print(arg.getValue());
	  if (arg != call.getArgs().get(call.getArgs().size() - 1))
		getOut().print(",");
	}
	getOut().print("]]");
  }

  public void walk(CnstrAction cnstr) {
	getOut().print("(Cnstr)[name=");
	getOut().print(cnstr.getConstructor().getName());
	getOut().print(",args=[");
	for (Str arg : cnstr.getArgs()) {
	  getOut().print(arg.getValue());
	  if (arg != cnstr.getArgs().get(cnstr.getArgs().size() - 1))
		getOut().print(",");
	}
	getOut().print("]]");
  }

  public void walk(Element element) {
	getOut().print("(Element)[tag=");
	getOut().print(element.getTag());
	getOut().print(",child=");
	walk(element.getChild());
	getOut().print(",atts=[");
	for (Str att : element.getNames()) {
	  getOut().print(att.getValue());
	  if (att != element.getNames().get(element.getNames().size() - 1))
		getOut().print(",");
	}
	getOut().print("]]");
  }

  public void walk(Star star) {
	getOut().print("(Star)[parser=");
	walk(star.getParser());
	getOut().print("]");
  }

  public void walk(VarRef varRef) {
	getOut().print("(VarRef)[name=" + varRef.getName() + "]");
  }

}
