package parse;

import xcore.List;
import xcore.Str;
import xcore.Value;
import env.Env;
import env.NullEnv;

public class Rule extends Value {

  // A rule is a named, parametric parser.

  private String name;
  private List<Str> args;
  private Parser body = new OK();

  public Rule(Str name) {
	this.name = name.getValue();
  }

  public Rule(Str name, List<Str> args) {
	this.name = name.getValue();
	this.args = args;
  }

  public Rule(Str name, List<Str> args, Parser body) {
	super();
	this.name = name.getValue();
	this.args = args;
	this.body = body;
  }

  public Rule(String name, List<Str> args, Parser... body) {
	super();
	this.name = name;
	this.args = args;
	this.body = consAnd(body);
  }

  private Parser consAnd(Parser[] body) {
	if (body.length == 0)
	  return new OK();
	else if (body.length == 1)
	  return body[0];
	else {
	  Parser parser = body[0];
	  for (int i = 1; i < body.length; i++)
		parser = new And(parser, body[i]);
	  return parser;
	}
  }

  public String getName() {
	return name;
  }

  public void setName(String name) {
	this.name = name;
  }

  public List<Str> getArgs() {
	return args;
  }

  public void setArgs(List<Str> args) {
	this.args = args;
  }

  public Parser getBody() {
	return body;
  }

  public void setBody(Parser body) {
	this.body = body;
  }

  public TagProcessor parse(Grammar grammar, Success s, List<Value> values, Fail c, Fail f) {
	// Bind the arguments and parse the body...
	Env<String, Value> env = new NullEnv<String, Value>();
	for (int i = 0; i < java.lang.Math.min(args.size(), values.size()); i++)
	  env = env.bind(args.get(i).getValue(), values.get(i));
	return body.parse(grammar, s, env, c, f);
  }

  public String toString() {
	String s = "Rule(" + name + "," + body + ")";
	return s;
  }
}
