package xcore;

import java.io.PrintStream;

public class Printer extends Walker {

  // A printer for xcore values. Deals with cycles and marks...

  private PrintStream out;

  public Printer(PrintStream out) {
	this.out = out;
  }

  public PrintStream getOut() {
	return out;
  }

  public void setOut(PrintStream out) {
	this.out = out;
  }

  public void walk(Bool bool) {
	out.print(bool.getValue());
  }

  public void walk(Cons<Value> cons) {
	out.print("(");
	walk(cons.getHead());
	out.print(".");
	walk(cons.getTail());
	out.print(")");
  }

  public void walk(Float f) {
	out.print(f.getValue());
  }

  public void walk(Int i) {
	out.print(i.getValue());
  }

  public void walk(Nil<Value> nil) {
	out.print("[]");
  }

  public void walk(Obj obj) {
	out.print("(");
	walk(obj.getType());
	out.print(")[");
	for (Slot s : obj.getSlots()) {
	  walk(s);
	  if (s != obj.getSlots().lastElement())
		out.print(",");
	}
	out.print("]");
  }

  public void walk(Ref ref) {
	out.print(ref.getId());
  }

  public void walk(Str str) {
	out.print(str.getValue());
  }

  public void rewalk(Value value) {
	if (getMark() != null && getMark().getMark(value) != -1)
	  out.print("$" + getMark().getMark(value));
	else
	  out.print("...");
  }

  public void walk(Slot slot) {
	out.print(slot.getName());
	out.print("=");
	walk(slot.getValue());
  }

  public void mark(Value v, int i) {
	getOut().print(i + "=");
  }

}
