package xcore;

import java.lang.reflect.InvocationTargetException;
import java.util.HashSet;

public abstract class Walker {

  // Used to walk values. The table is used to detect cycles. The
  // mark walker is used to allocate labels.

  private HashSet<Value> table = new HashSet<Value>();
  private Mark mark = null;

  public Mark getMark() {
	return mark;
  }

  public void setMark(Mark mark) {
	this.mark = mark;
  }

  public abstract void mark(Value v, int i);

  public void walk(Value v) {
	if (table.contains(v))
	  rewalk(v);
	else {
	  if (mark != null && mark.getMark(v) != -1)
		mark(v, mark.getMark(v));
	  if (v != null) {
		table.add(v);
		Object[] os = { v };
		Class<?> vClass = v.getClass();
		Class<?>[] cs = { vClass };
		try {
		  this.getClass().getMethod("walk", cs).invoke(this, os);
		} catch (java.lang.NoSuchMethodException e) {
		  throw new Error("No walk method for " + v + ":" + v.getClass() + " in " + this + ":" + getClass());
		} catch (IllegalArgumentException e) {
		  e.printStackTrace();
		} catch (SecurityException e) {
		  e.printStackTrace();
		} catch (IllegalAccessException e) {
		  e.printStackTrace();
		} catch (InvocationTargetException e) {
		  e.printStackTrace();
		}
	  }
	}
  }

  public abstract void rewalk(Value value);
}
